﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using System.Web;
using Microsoft.SharePoint.Administration;
using System.Diagnostics;
using System.Reflection;
using Microsoft.SharePoint;
using System.Web.Configuration;
using System.Configuration;
using System.Web.Security;
using Microsoft.SharePoint.Utilities;
using System.Collections;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Xsl;
using System.IO;
using System.Data;
using System.Text.RegularExpressions;
using System.Security.Principal;
using System.Collections.Specialized;
using System.Globalization;
using System.Net;

namespace ElevatedPrivilage
{
    public class Utils
    {
       
        /// <summary>Returns an elevated site</summary>
        /// <param name="theSite">
        /// The site that you want an elevated instance of.
        /// You must dispose of this object unless it is part of SPContext.Current.
        /// </param>
        /// <returns>An elevated site context.</returns>
        /// <remarks>Be sure to dispose of objects created from this method.</remarks>
        /// Usage:
        /// using (var site = Utils.GetElevatedSite(new SPSite(SPContext.Current.Web.Url)))
        //{
        //    // This code runs under the security context of the SHAREPOINT\system
        //    //  for all objects accessed through the "site" reference. Note that it's a 
        //    // different reference than SPContext.Current.Site. 
        //    using (var elevatedWeb = site.OpenWeb())
        //    {
        //      Perform actions as SYSTEM here
        //    }
        //}
        public static SPSite GetElevatedSite(SPSite theSite)
        {
            var sysToken = GetSystemToken(theSite);
            return new SPSite(theSite.ID, sysToken);
        }

        /// <summary>Gets a UserToken for the system account.</summary>
        /// <param name="site"></param>
        /// <returns>A usertoken for the system account user./returns>
        /// <remarks>Use this token to impersonate the system account</remarks>
        public static SPUserToken GetSystemToken(SPSite site)
        {
            site.CatchAccessDeniedException = false;
            try
            {
                return site.SystemAccount.UserToken;
            }
            catch (UnauthorizedAccessException)
            {
                SPUserToken sysToken = null;

                // Only use runwithelevated to grab the system user token.
                SPSecurity.RunWithElevatedPrivileges(
                    delegate()
                    {
                        using (SPSite killerSiteCol = new SPSite(site.ID))
                        {
                            sysToken = killerSiteCol.SystemAccount.UserToken;
                        }
                    }
                );
                return sysToken;
            }
        }      
    }
}
