using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

// SharePoint 2007 Dispose Patterns By Example
// SharePoint Dispose Microsoft Best Practices Examples http://blogs.msdn.com/rogerla
// By Roger Lamb, Architect ADC | Microsoft Corporation

namespace SPDisposeExamples
{
    public class SPSiteCollectionLeak
    {
        #region SPSiteCollection.Add()
        
        public void SPSiteCollectionAddLeak()
        {
            using (SPSite siteCollectionOuter = new SPSite("http://dhdemo:5551"))
            {
                SPWebApplication webApp = siteCollectionOuter.WebApplication;
                SPSiteCollection siteCollections = webApp.Sites;
                SPSite siteCollectionInner = siteCollections.Add("/myNewSiteCollection",
                    "dhdemo\\administrator", "admin@dhdemo.com");
                // SPSite siteCollectionInner leak
            } // SPSite object siteCollectionOuter.Dispose() automatically called
        }

        public void SPSiteCollectionAddNoLeak()
        {
            using (SPSite siteCollectionOuter = new SPSite("http://moss"))
            {
                SPWebApplication webApp = siteCollectionOuter.WebApplication;
                SPSiteCollection siteCollections = webApp.Sites;
                using (SPSite siteCollectionInner = siteCollections.Add("/myNewSiteCollection",
                    "dhdemo\\administrator", "admin@dhdemo.com"))
                {
                } // SPSite object siteCollectionInner.Dispose() automatically called
            } // SPSite object siteCollectionOuter.Dispose() automatically called
        }
        #endregion

        #region SPSiteCollectionIndexer

        //[SPDisposeCheckIgnoreAttribute(SPDisposeCheckID.SPDisposeCheckID_230, "Don't want to do it")]
        
        public void SPSiteCollectionIndexerLeak()
        {
            using (SPSite siteCollectionOuter = new SPSite("http://dhdemo:5551"))
            {
                SPWebApplication webApp = siteCollectionOuter.WebApplication;
                SPSiteCollection siteCollections = webApp.Sites;

                SPSite siteCollectionInner = siteCollections[0];
                // SPSite siteCollectionInner leak 
            } // SPSite object siteCollectionOuter.Dispose() automatically called
        }

        public void SPSiteCollectionIndexerNoLeak()
        {
            using (SPSite siteCollectionOuter = new SPSite("http://dhdemo:5551"))
            {
                SPSite siteCollectionInner = null;
                try
                {
                    SPWebApplication webApp = siteCollectionOuter.WebApplication;
                    SPSiteCollection siteCollections = webApp.Sites;
                    // [Reza] There is also another issue with the following line of code
                    // which will be addressed in performance slide
                    siteCollectionInner = siteCollections[0];
                }
                finally
                {
                    if (siteCollectionInner != null)
                        siteCollectionInner.Dispose();
                }
            } // SPSite object siteCollectionOuter.Dispose() automatically called
        }      

        #endregion
    }
}
